'use strict';Object.defineProperty(exports, "__esModule", { value: true });exports.default = void 0;

var _cron = require("cron");
var _child_process = require("child_process");
var _debounce = _interopRequireDefault(require("debounce"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);Object.defineProperty(Constructor, "prototype", { writable: false });return Constructor;}var

MonitorService = /*#__PURE__*/function () {
  function MonitorService(_ref) {var _this = this;var getServices = _ref.getServices,logger = _ref.logger,dataHandler = _ref.dataHandler,notificationHandler = _ref.notificationHandler,privateSocket = _ref.privateSocket,publicSockets = _ref.publicSockets;_classCallCheck(this, MonitorService);
    this.logger = logger;

    this.cronDbBackup = new _cron.CronJob(
    '20 56 1 * * 1,4', // each mondays & thursdays, 1:56:20
    this.backupDb,
    null, true, 'Europe/Paris', this);


    this.cronRaspberryTemp = new _cron.CronJob(
    '45 * * * * *', // each minute, 45s
    this.raspberryTemp,
    null, true, undefined, this);

    this.raspberryTempErrors = 0;
    this.debouncerRaspberryTempWarning = (0, _debounce.default)(function (temp) {
      notificationHandler.pushNotificationMessage(
      'Asterism Monitoring',
      `Raspberry Pi temperature high (${temp}°C).`,
      'warning',
      { tag: 'monitor-temperature-1', renotify: true });

    }, 15 * 60000, true);

    setTimeout(function () {
      _this.cronCpuLoad = new _cron.CronJob(
      '*/30 * * * * *', // each 30s
      _this.cpuLoad,
      null, true, undefined, _this);

    }.bind(this), 5 * 60000);
    this.cpuLoadErrors = 0;
    this.cpuLoads = [];
    this.debouncerCpuLoadWarning = (0, _debounce.default)(function (mean) {
      notificationHandler.pushNotificationMessage(
      'Asterism Monitoring',
      `CPU load high (${mean}%).`,
      'warning',
      { tag: 'monitor-cpu-1', renotify: true });

    }, 5 * 60000, true);
  }_createClass(MonitorService, [{ key: "backupDb", value:

    function backupDb() {var _this2 = this;
      this.logger.log('Monitor:DbBackup...', new Date().toString());
      (0, _child_process.exec)(
      `tar -cvzf "./var/backup/auto/$(date '+%Y%m%d-%H%M%S').db.tgz" ./var/*.nosql`,
      function (err, stdout, stderr) {
        if (err || stderr) {
          _this2.logger.error('Backuping DB to tgz failed!');
          return;
        }
        (0, _child_process.exec)(
        'find ./var/backup/auto/*.tgz -type f -mmin +86400 -delete',
        function (err2, stdout2, stderr2) {
          if (err2 || stderr2) {
            _this2.logger.error('Cleaning backups DB failed!');
            return;
          }
        });

      });

    } }, { key: "raspberryTemp", value:

    function raspberryTemp() {var _this3 = this;
      (0, _child_process.exec)(
      'cat /sys/class/thermal/thermal_zone0/temp',
      function (err, stdout, stderr) {
        if (err || stderr) {
          _this3.raspberryTempErrors++;
          if (_this3.raspberryTempErrors > 5) {var _this3$cronRaspberryT;
            _this3.logger.warn('RaspberryTemp monitor failed 5 times. Monitor canceled!');
            _this3 === null || _this3 === void 0 ? void 0 : (_this3$cronRaspberryT = _this3.cronRaspberryTemp) === null || _this3$cronRaspberryT === void 0 ? void 0 : _this3$cronRaspberryT.stop();
          }
          return;
        }
        try {
          var temperature = Math.trunc(Number.parseInt(stdout, 10) / 1000);
          if (temperature > 45) {
            _this3.logger.warn(`Monitor/raspberryTemp: ${temperature} is high. Sending notification...`);
            _this3.debouncerRaspberryTempWarning(temperature);
          }
          if (temperature > 55) {
            _this3.logger.error(`Monitor/raspberryTemp: ${temperature} is too high. Rebooting node process...`);
            // TODO !0: reboot requis d'urgence: exit de node permettra à pm2 de relancer
          }
        } catch (err2) {
          _this3.logger.error(err2);
        }
      });

    } }, { key: "cpuLoad", value:

    function cpuLoad() {var _this4 = this;
      (0, _child_process.exec)(
      'pm2 jlist',
      function (err, stdout, stderr) {
        if (err || stderr) {
          _this4.cpuLoadErrors++;
          if (_this4.cpuLoadErrors > 5) {var _this4$cronCpuLoad;
            _this4.logger.warn('CpuLoad monitor failed 5 times. Monitor canceled!');
            _this4 === null || _this4 === void 0 ? void 0 : (_this4$cronCpuLoad = _this4.cronCpuLoad) === null || _this4$cronCpuLoad === void 0 ? void 0 : _this4$cronCpuLoad.stop();
          }
          return;
        }
        try {
          var dump = JSON.parse(stdout);
          var cpus = dump.map(function (line) {return { name: line.name, cpu: line.monit.cpu, memory: line.monit.memory };});
          var sum = cpus.reduce(function (acc, _ref2) {var cpu = _ref2.cpu;return acc + cpu;}, 0);
          _this4.cpuLoads.push(sum);
          while (_this4.cpuLoads.length > 30) {
            _this4.cpuLoads.splice(0, 1);
          }
          if (_this4.cpuLoads.length < 28) {
            return;
          }
          var mean8 = _this4.cpuLoads.slice(-8).reduce(function (acc, v) {return acc + v;}, 0) / 8;
          var mean28 = _this4.cpuLoads.slice(-28).reduce(function (acc, v) {return acc + v;}, 0) / 28;

          if (mean8 > 80) {
            _this4.logger.warn(`Monitor/cpuLoad: CPU over 80% for 4 minutes. Sending notification...`);
            _this4.debouncerCpuLoadWarning(mean8);
          }
          if (mean28 > 90) {
            _this4.logger.warn(`Monitor/cpuLoad: CPU over 90% for 14 minutes. Rebooting node process...`);
            // TODO !0: reboot requis d'urgence: exit de node permettra à pm2 de relancer
          }
        } catch (err2) {
          _this4.logger.error(err2);
        }
      });


    }

    // TODO !1: monitor zwave key stability (toutes les 5 minutes) :
    //  > ls -la /dev/tty##### (ACM0 ou autre, selon la BDD) | wc -l
    //  les periphs doivent rester constants. en cas d'erreur, notification (pas de reboot, inutile)
  }, { key: "destroy", value:
    function destroy() {var _this$cronDbBackup, _this$cronRaspberryTe, _this$cronCpuLoad;
      this === null || this === void 0 ? void 0 : (_this$cronDbBackup = this.cronDbBackup) === null || _this$cronDbBackup === void 0 ? void 0 : _this$cronDbBackup.stop();
      this === null || this === void 0 ? void 0 : (_this$cronRaspberryTe = this.cronRaspberryTemp) === null || _this$cronRaspberryTe === void 0 ? void 0 : _this$cronRaspberryTe.stop();
      this === null || this === void 0 ? void 0 : (_this$cronCpuLoad = this.cronCpuLoad) === null || _this$cronCpuLoad === void 0 ? void 0 : _this$cronCpuLoad.stop();
    } }]);return MonitorService;}();exports.default = MonitorService;